import {createRouter, createWebHashHistory} from "vue-router";

import route_vue_get from "../route/getVue.js";

import storage from "../tool/storage.js";

import {createHead} from '@unhead/vue';

import alone from "../tool/alone.js";

import {createI18n} from "vue-i18n";

import safe from "../tool/safe.js";

import NProgress from "nprogress";

import Apps from "@src/App.vue";

import {createApp} from "vue";

import app from "./app.js";

import axios from "axios";

export const create = createApp(Apps);

const self = {
    //本地设置的路由,和远程路由
    route: {local: [], web: []}, plugins(key) {
        return alone.get_arr(process.plugins, key);
    }, //加载本地配置
    local_conf() {
        const local = process.config || {data: null, random: null};
        const conf = ((local.data || '' && local.random || '') ? (safe.mov.de(local) || local) : local);
        if (alone.isset(conf)) {
            this.route.local = conf.route || [];
            app.config.object(alone.del_key(conf, 'route'));
        }
        return this;
    },
    async axios(path) {
        axios.defaults.withCredentials = true;
        return (await axios.get(path, {
            timeout: 10000,
            baseURL: app.get_env('api'),
            headers: {dev_mode: import.meta.env.MODE == 'development' ? "dev" : "prod"},
            params: {
                data: safe.url.en({
                    make: app.get_make(), origin: app.get_origin(), domain: alone.domain.domain()
                })
            }
        }).then(function (data) {
            return data.data;
        }, function (data) {
            return data.data;
        }));
    },
    async http(path) {
        return await app.http(path)
            .query({
                data: safe.url.en({
                    make: app.get_make(), origin: app.get_origin(), domain: alone.domain.domain()
                })
            })
            .header({dev_mode: import.meta.env.MODE == 'development' ? "dev" : "prod"})
            .get(function (res) {
                return res;
            }, function () {
                return false;
            });
    },
    //加载远程配置
    async web_conf() {
        const conf_file = app.get_env('conf');
        if (alone.isset(conf_file) && alone.is_string(conf_file)) {
            let res = app.get_env('axios') !== false ? (await this.axios(conf_file)) : (await this.http(conf_file));
            let data = res;
            if (alone.isset(res)) {
                if ((res?.data || '') && (res?.random || '')) {
                    data = (safe.mov.de(res) || res);
                    if (data.code == 200 && alone.isset(data.data)) {
                        data = data.data;
                    }
                }
                if (alone.isset(data)) {
                    this.route.web = data.route || [];
                    app.config.object(alone.del_key(data, 'route'));
                }
            }
        }
        return this;
    },
    async process() {
        //有效时间
        const make_time = parseInt(app.config.get('make_time', 28800));
        //生成访问标识
        if (!alone.isset(app.get_make())) {
            storage.set('makeCache', alone.get_token(), make_time);
        }
        //保存第一次的来路
        if (!alone.isset(app.get_origin())) {
            const origin = alone.origin.get();
            if (alone.isset(origin)) {
                storage.set('originCache', origin, make_time);
            }
        }
        this.local_conf();
        await this.web_conf();
        app.set_title();
        return this;
    }, //处理路由
    load_route(view) {
        this.route_list = route_vue_get(Object.assign(app.config.get('routing', {}) || {}, {view: view}), this.route.local || [], this.route.web || []);
        app.config.list().route = this.route_list.route;
        this.route_config = this.route_list.routes || [];
        return this;
    }, load_i18n() {
        this.i18n_config = {
            locale: app.config.get('locale'), fallback: app.config.get('fallback'), messages: app.config.get('lang'),
        };
        return this;
    }, start_route(array) {
        if (alone.isset(array)) {
            app.routes = (alone.is_object(array) ? Object.values(array) : array);
            app.route = createRouter({history: createWebHashHistory(import.meta.env.BASE_URL), routes: app.routes});
            const black = ' style="display: block"';
            const none = ' style="display: none"';
            const bar = (app.config.get('load_bar') == true || app.config.get('load_bar') == 1) ? black : none;
            const ico = (app.config.get('load_ico') == true || app.config.get('load_ico') == 1) ? black : none;
            //启动 进度条和加载图标
            const bar_start = () => NProgress.start();
            //关闭 进度条和加载图标
            const bar_close = () => NProgress.done();
            NProgress.settings.template = `<div class="bar" role="bar"${bar}><div class="peg"></div></div><div class="spinner" role="spinner"${ico}><div class="spinner-icon"></div></div>`;
            app.route.beforeEach((to, from, next) => {
                const title = alone.get_arr(to, 'meta.title', null);
                if (alone.isset(title)) {
                    app.set_title(title);
                }
                if (to.matched.length) {
                    bar_start();
                    next();
                } else {
                    const error = app.config.get('error');
                    if (!error) {
                        return false;
                    }
                    next(error);
                }
            });
            app.route.afterEach(() => (bar_close()));
            app.route.onError(() => (bar_close()));
            create.use(app.route);
        }
        return this;
    },
    start_i18n({locale, fallback, messages} = {}) {
        if (alone.isset(locale)) {
            locale = storage.get('langLocale') || locale;
            app.i18n = createI18n({locale: locale, fallbackLocale: fallback || locale, messages: messages});
            app.get_locale(locale, fallback);
            create.use(app.i18n);
        }
        return this;
    },
    async main(view) {
        await this.process();
        this.load_route(view).load_i18n();
        this.start_route(this.route_config).start_i18n(this.i18n_config);
        import('vfonts/Lato.css');
        import('vfonts/FiraCode.css');
        import('../style/css/bar.css');
        return this;
    }
};
export default {
    app: Apps, create: create, createApp: createApp, status: {}, use(plugin) {
        return create.use(plugin);
    }, //加载前
    async load(view = {}) {
        if (!this.status.load) {
            this.status.load = true;
            if (self.plugins('markdown')) {
                this.use(createHead());
            }
            await self.main(view);
        }
        return self;
    }, //加载后
    mount() {
        if (!this.status.mount) {
            this.status.mount = true;
            this.create.mount('#' + app.get_env('app'));
        }
        return this.create;
    }
}